// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TrueTypeLoader.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

TrueTypeLoader::TrueTypeLoader(const std::string name) :
  AssetLoader(name),
  loader(name + ".ttf")
{

}

void TrueTypeLoader::updateSync() {
}

void TrueTypeLoader::updateAsync() {
  if(state != TrueTypeLoaderState::INITIAL) return;
  state = TrueTypeLoaderState::ASYNC_LOADING;

  // Load the data.
  size_t size = loader.getSize();
  buffer = new uint8_t[size];

  // Read the data.
  size_t readSize = loader.read(buffer, size);
  assertTrue(readSize == size, "Failed to read all data from TrueTypeLoader.");

  // Init the font.
  auto ret = FT_Init_FreeType(&fontLibrary);
  assertTrue(ret == 0, "Failed to initialize FreeType library.");

  ret = FT_New_Memory_Face(fontLibrary, buffer, size, 0, &face);
  assertTrue(ret == 0, "Failed to load font face.");

  // Now close the asset loader
  loader.close();
  state = TrueTypeLoaderState::ASYNC_DONE;
  this->loaded = true;
}

std::shared_ptr<TrueTypeTexture> TrueTypeLoader::getTexture(
  const uint32_t fontSize
) {
  // Check if we have the texture already and it hasn't gone stale.
  auto it = textures.find(fontSize);
  if(it != textures.end()) {
    if(!it->second.expired()) return it->second.lock();
    textures.erase(it);
  }

  // Create the texture.
  auto texture = std::make_shared<TrueTypeTexture>(face, fontSize);
  textures[fontSize] = texture;
  return texture;
}

TrueTypeLoader::~TrueTypeLoader() {
  if(buffer != nullptr) {
    delete[] buffer;
    buffer = nullptr;
  }
}